/*********************************************************************************************
 * File:    IndMap.java (Primary activity)
 * Project: Indoor maps, Masterthesis
 * Version: 0.5b
 * Author:  Jon Villy Meidell
 * Org:     NTNU
 * 
 * Scale for drawing: 1 meter = 44
 */

package no.ntnu.map.indoormaps;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.MissingResourceException;
import java.util.TreeSet;

import no.ntnu.map.indoormaps.NodeActivity.ApListCursorAdapter;
import no.ntnu.map.indoormaps.NodeActivity.ScanResultLevelComparator;
import no.ntnu.map.indoormaps.Point.RotatePoint;
import no.ntnu.map.indoormaps.Storage.MysqlViaPHP;
import android.app.Activity;
import android.app.AlertDialog;
import android.app.Dialog;
import android.app.ProgressDialog;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.SharedPreferences;
import android.database.MatrixCursor;
import android.graphics.PointF;
import android.net.Uri;
import android.net.wifi.ScanResult;
import android.net.wifi.WifiManager;
import android.opengl.GLSurfaceView;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.util.Log;
import android.view.KeyEvent;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.MotionEvent;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.Window;
import android.widget.AdapterView;
import android.widget.Button;
import android.widget.EditText;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.SimpleCursorAdapter;
import android.widget.TextView;
import android.widget.Toast;
import android.widget.AdapterView.OnItemClickListener;

/**
 * The Class IndMap.
 * <br/><br/>
 * Part of Grafical User Interface - 
 * The main activity file.
 * 
 * 
 * <p>Intent filters:
 * <ul>
 *   <li>indmap:*</li>
 * 
 * </ul>
 * 
 */
public class IndMap extends Activity
{

	/** TAG used for logcat. */
	public static final String TAG = "IndMap";

	/** For debug purposes. */
	public static final boolean D = false;
	/** For debugging loading from {@link World.LFEThread external storage}. */
	public static final boolean LOAD_EX_DEBUG=false;
	/** For debugging loading from {@link World#loadFromInternal(IndMap) internal storage}. */
	public static final boolean LOAD_INT_DEBUG=false;
	/** For debugging a Wifi scan {@link #receiver result}. */
	public static final boolean FIND_AT_NODE_DEBUG=true;

	/** For debugging the moving between {@link Node nodes} and within a {@link Route}. */
	public static final boolean MOVE_DEBUG=false;

	/** The PREFERENCES settings file. */
	private static final String	PREFERENCES	= "indmap";

	/** The navigation buttons states. */
	private static final short MODE_ROT=0;

	/** The navigation buttons states. */
	private static final short	MODE_Z=1;
	//	private static final short[] BUTTON_MODE=new short[]{MODE_ROT,MODE_Z};

	/** Wifi-search state. @uml.property  name="wIFI_NONE" */
	private final short WIFI_NONE=0;

	/** Wifi-search state. @uml.property  name="wIFI_WIFI" */
	private final short WIFI_WIFI=1;

	/**
	 * A Comparator for sorting the scan results ascending on signal level of different {@link Ap#getNodes() nodes} by an Access Point.
	 *
	 * @see IndMap#receiver
	 * @see android.net.wifi.ScanResult
	 */
	private static class NodeApScanComparator implements Comparator<Node>
	{

		/** Hashmap for easy access level for a {@link Node node} from these {@link Ap#getNodes() nodes}. */
		private HashMap<Node, Integer> hm=new HashMap<Node, Integer>(30);

		/**
		 * Detect if the NodeApScanCOmparator has a level record for given {@link Node Node}.
		 *
		 * @param key The {@link Node Node}
		 * @return true, if {@link Node Node} is present
		 */
		public boolean contains(Node key){return hm.containsKey(key);}

		/**
		 * For adding level value for a {@link Node Node}.
		 *
		 * @param key The {@link Node Node}
		 * @param val level
		 */
		public void add(Node key,Integer val){hm.put(key, val);}

		/**
		 * Gets a {@link Node Nodes} level.
		 *
		 * @param key the key
		 * @return the value
		 */
		public int getValue(Node key){return hm.get(key).intValue();}

		/**
		 * Compares the levels for the nodes by the {@link #hm HashMap}.
		 *
		 * @param a the a
		 * @param b the b
		 * @return the int
		 * @see java.util.Comparator#compare(java.lang.Object, java.lang.Object)
		 */
		public int compare(Node a, Node b)
		{
			if(!hm.containsKey(a))throw new MissingResourceException("Node not found. Needed for comparation", "NodeApScanComparator", a.toString());
			if(!hm.containsKey(b))throw new MissingResourceException("Node not found. Needed for comparation", "NodeApScanComparator", b.toString());
			return hm.get(a)-hm.get(b);
		}
	}

	/**
	 * The navigation button mode.  {@link #MODE_ROT  rotate}  and  {@link #MODE_Z  z} 
	 * @uml.property  name="buttonMode"
	 */
	private short buttonMode=MODE_ROT;

	/**
	 * For access to datastructure.
	 * @uml.property  name="world"
	 * @uml.associationEnd  
	 */
	private World world;

	/** For access to external storage http/mysql. @uml.property  name="myphp" @uml.associationEnd */
	private MysqlViaPHP myphp;

	/** The GLSurface to display graphics. @uml.property  name="glSurfaceView" @uml.associationEnd */
	private GLSurfaceView glSurfaceView;

	/**
	 * The touch start for use with  {@link #onTouchEvent(MotionEvent)  this} .
	 * @uml.property  name="touchStart"
	 * @uml.associationEnd  
	 */
	private PointF touchStart;

	/**
	 * The progress dialog for loading from external.
	 * @uml.property  name="progressDialog"
	 * @uml.associationEnd  
	 */
	private ProgressDialog progressDialog;

	/** The thread for external loading, opened from  {@link #onPrepareDialog(int,Dialog)}. @uml.property  name="lfeThread" @uml.associationEnd */
	private World.LFEThread lfeThread;

	/** The open uri, given in Intent starting the activity. @uml.property  name="openUri" @uml.associationEnd */
	private Uri openUri;

	/** The ap scan result list. */
	private Dialog apScanResultList;

	/**
	 * Handler to Wifi Manager.
	 * @uml.property  name="wiMan"
	 * @uml.associationEnd  
	 */
	private WifiManager wiMan;

	/** Keeper for current wifi-state:  {@link #WIFI_NONE  none}  or  {@link #WIFI_WIFI  wifi}. @uml.property  name="receive_for" */
	private short receive_for=WIFI_NONE;

	/**
	 * The receiver for handlig  {@link #wiMan  Wifi}  scan results Uses the best  {@link ScanResult  ScanResults} , sorted by the  {@link ScanResultLevelComparator  best signal level} , finding the a matching  {@link Ap  Access Point}  from  {@link World#getAps()  known Ap's} . Then calculate a score for each  {@link Node  node}  given in the found  {@link Ap  Access Point's}   {@link Ap#getNodes()  nodes}  based on the  {@link Node#getAPs()  nodes known AP's}  and the according  {@link Node#getAPLevels()  signal level} An   {@link ApListCursorAdapter  custom Adapter}  keeps the ScanResults for displating in a  {@link android.widget.ListView  ListView} A  {@link TreeSet  TreeSet}  keeps the found nodes and level-scores by  {@link NodeApScanComparator  this}  Comparator.
	 * @see ScanResult  
	 * @see  Comparator
	 * @uml.property  name="receiver"
	 * @uml.associationEnd  multiplicity="(1 1)"
	 */
	private BroadcastReceiver receiver=new BroadcastReceiver()
	{
		@Override
		public void onReceive(Context context, Intent intent)
		{
			if(receive_for==WIFI_NONE)return;

			try{
				if(FIND_AT_NODE_DEBUG)IndMap.log(IndMap.DEBUG, TAG, "Wifi ***************************************************");
				List<ScanResult> srs=wiMan.getScanResults();
				Collections.sort(srs,new ScanResultLevelComparator());
				ApListCursorAdapter apl=new ApListCursorAdapter(getApplicationContext(), R.layout.aplist_ttlp_item, srs, world.getAps(), false,true);

				final int MISSING_AP_COST=100,MAX_NODES_RESULT=10;
				Ap[] foundAps=apl.getAps();
				NodeApScanComparator nasc=new NodeApScanComparator();
				TreeSet<Node> ts=new TreeSet<Node>(nasc);

				for(int h=0;h<apl.size();h++)
				{
					Ap a=apl.getAp(h);
					if(FIND_AT_NODE_DEBUG)IndMap.log(IndMap.DEBUG, TAG, "Wifi #"+(h+1)+" AP found: "+a.toString());

					ArrayList<Node> no=a.getNodes();
					if(FIND_AT_NODE_DEBUG)IndMap.log(IndMap.DEBUG, TAG, "Wifi   - The ap got "+no.size()+" nodes");

					for(int i=0;i<no.size();i++) //Iterate through nodes at best Ap
					{
						Node n=no.get(i);
						if(FIND_AT_NODE_DEBUG)IndMap.log(IndMap.DEBUG, TAG, "Wifi   - Node #"+(i+1)+": "+n.toString());
						if(nasc.contains(n))continue;
						Ap[] naps=n.getAPs();
						int sum=MISSING_AP_COST*naps.length;
						if(FIND_AT_NODE_DEBUG)IndMap.log(IndMap.DEBUG, TAG, "Wifi     - Node got "+naps.length+" aps linked");
						for(int j=0;j<naps.length;j++)
						{
							int index=-1,id=naps[j].getI();
							for(int k=0;k<foundAps.length;k++)
							{
								if(foundAps[k].getI()==id)
								{
									index=k;
									break;
								}
							}
							if(FIND_AT_NODE_DEBUG)IndMap.log(IndMap.DEBUG, TAG, "Wifi       - Ap index: "+index);
							if(index<0)continue;

							int l1=apl.getLevel(index),
							l2=n.getAPLevels()[j];
							if(FIND_AT_NODE_DEBUG)IndMap.log(IndMap.DEBUG, TAG, "Wifi       - L1: "+apl.getAp(index).toString()+" - "+l1);
							if(FIND_AT_NODE_DEBUG)IndMap.log(IndMap.DEBUG, TAG, "Wifi       - L2: "+naps[j].toString()+" - "+l2);
							sum+=(l1-l2)*(l1-l2)- MISSING_AP_COST;
						}
						if(FIND_AT_NODE_DEBUG)IndMap.log(IndMap.DEBUG, TAG, "Wifi       SUM: "+sum);

						nasc.add(n, new Integer(sum));
						ts.add(n);
					}
					if(FIND_AT_NODE_DEBUG)IndMap.log(IndMap.DEBUG, TAG, "Wifi   - Having: "+ts.size()+" nodes");

				}
				//BUGS
				/*
					if(ts.size()>MAX_NODES_RESULT*2)
					{
						a=apl.getAp(1);

						if(FIND_AT_NODE_DEBUG)IndMap.log(IndMap.DEBUG, TAG, "To many, so second closest AP: "+a.toString());
						no=a.getNodes();
						tmp=new TreeSet<Node>();
						tmp.addAll(ts);
						tmp.addAll(no);
						if(tmp.size()>MIN_NODES_RESULT)ts=tmp;
					}*/

				Dialog d=apScanResultList=new Dialog(IndMap.this);
				d.setContentView(R.layout.apnode_lists);
				d.setTitle(R.string.ap_found);
				ListView lvAps=(ListView) d.findViewById(R.id.aps);
				IndMap.log(IndMap.DEBUG, TAG, "APs list "+(lvAps==null?"null":"found"));
				lvAps.setAdapter(apl);

				ListView lvNodes=(ListView) d.findViewById(R.id.nodes);
				IndMap.log(IndMap.DEBUG, TAG, "Nodes list "+(lvNodes==null?"null":"found"));
				final String[] cols=new String[]{"_id","b","c"};
				MatrixCursor c=new MatrixCursor(cols, ts.size());
				Iterator<Node> it=ts.iterator();
				int i=0;
				while(it.hasNext()&&i<MAX_NODES_RESULT)
				{
					Node n=it.next();
					c.addRow(new String[]{""+n.getI(),n.getName()+" ("+n.getDesc()+")", ""+nasc.getValue(n)});
					i++;
				}
				lvNodes.setAdapter(new SimpleCursorAdapter(getApplicationContext(), R.layout.list_3tv_item, c, cols, new int[]{R.id.tv1,R.id.tv2,R.id.tv3}));
				lvNodes.setOnItemClickListener(new OnItemClickListener()
				{
					public void onItemClick(AdapterView<?> parent, View view, int position, long id)
					{
						Node n=(Node) Node.searchById(world.getNodes(),(int) id);
						Route r=world.getAtRoute();
						if(r!=null)
						{
							if(r.isMember(n))
							{
								world.setAtNode(n);
								apScanResultList.dismiss();
							}
							else Toast.makeText(getApplicationContext(), "The node is not part of the active route..", Toast.LENGTH_LONG).show();
						}
						else 
						{
							world.setAtNode(n);
							apScanResultList.dismiss();
						}
					}
				});
				d.show();

			}catch (Exception e) {
				IndMap.log(IndMap.ERROR, TAG, "Show scan "+e.toString());
			}
			receive_for=WIFI_NONE;
		}
	};

	/**
	 * Initiates the activity, loads preferences, sets up the {@link IndMap#world this} {@link World world} ect.
	 *
	 * @param savedInstanceState the saved instance state
	 * @see android.app.Activity#onCreate(android.os.Bundle)
	 */
	public void onCreate(Bundle savedInstanceState)
	{
		super.onCreate(savedInstanceState);
		requestWindowFeature(Window.FEATURE_NO_TITLE);
		wiMan=(WifiManager) getSystemService(WIFI_SERVICE);

		//final Object data = getLastNonConfigurationInstance(); //Om dataene blir for store slik at det gr treigt  bytte orientation p skjerm
		world=new World();
		touchStart = new PointF();

		try
		{
			//Load preferences
			SharedPreferences set = getSharedPreferences(PREFERENCES, 0);
			myphp=new MysqlViaPHP(set.getString("myphp", "http://192.168.0.10/api/"));
			//			sidebar=(short)set.getInt("sidebar", View.VISIBLE);
			RotatePoint p=new RotatePoint(
					set.getFloat("X", 0.0f),
					set.getFloat("Y", 0.0f),
					set.getFloat("Z", 0.0f),
					0.0f,
					set.getFloat("UpRot", 0.0f),
					set.getFloat("UpSide", 0.0f)
			);
			if(p.y>40f||p.y<-40f)p.y=0.0f;
			world.setCamera(p);
			world.loadFromInternal(this);

			openUri=getIntent().getData();
			if(openUri!=null)
			{
				IndMap.log(IndMap.DEBUG, TAG, "Opened by URI: "+openUri.toString()+"  "+openUri.getScheme());
				try{
					if(openUri.getScheme().equals("indmap"))
					{
						IndMap.log(IndMap.INFO, TAG, "Opening: "+openUri.toString());
						String[] r=openUri.toString().split(":");
						if(r.length>2)
						{
							if(r[1].equals("node"))world.setAtNode(Integer.parseInt(r[2]));
							if(r[1].equals("asRoute"))
							{
								String[] ids=r[2].split("&");
								int[] id=new int[ids.length];
								for(int i=0;i<ids.length;i++)id[i]=Integer.parseInt(ids[i]);
								world.setAtRoute(new Route(world.getNodes(), id, false));

								//world.setAtRoute(new Route(world.getNodes(), b.getIntArray("asRoute"), false));
							}
							if(r[1].equals("goto")&&r.length>7)
							{
								world.moveTo(new RotatePoint(
										Float.parseFloat(r[2]), 
										Float.parseFloat(r[3]), 
										Float.parseFloat(r[4]), 
										0f, 
										Float.parseFloat(r[6]), 
										Float.parseFloat(r[7])
								), World.TRANS_ALL);
							}
						}
					}
				}catch (Exception e) {
					IndMap.log(IndMap.ERROR, TAG, "URI open FAIL "+e.toString());
				}
			}
			else openUri=Uri.parse("indmap:");

			final Bundle b=getIntent().getExtras();
			checkBundle(b);

			loadView();
		}
		catch (Exception e) {
			IndMap.log(IndMap.ERROR, TAG, "Startup "+e.toString());
		}
	}


	/**
	 * On resume.
	 *
	 * @see android.app.Activity#onResume()
	 */
	public void onResume()
	{
		super.onResume();
		registerReceiver(receiver, new IntentFilter(WifiManager.SCAN_RESULTS_AVAILABLE_ACTION));
		glSurfaceView.onResume();
	}

	/**
	 * On pause.
	 *
	 * @see android.app.Activity#onPause()
	 */
	protected void onPause() {
		super.onPause();
		unregisterReceiver(receiver);
		glSurfaceView.onPause();
	}

	/**
	 * Saves preferences when activity is stopped.
	 *
	 * @see android.app.Activity#onStop()
	 */

	public void onStop()
	{
		super.onStop();

		IndMap.log(IndMap.DEBUG, TAG, "Exiting, so saving perferences");
		//Save preferences
		SharedPreferences set = getSharedPreferences(PREFERENCES, 0);
		SharedPreferences.Editor e = set.edit();
		e.putString("myphp", myphp.getURL());
		//		e.putInt("view", currentView);
		RotatePoint p=world.getCamera();
		e.putFloat("X", p.x);
		e.putFloat("Y", p.y);
		e.putFloat("Z", p.z);
		e.putFloat("UpRot", p.rotate_y);
		e.putFloat("UpSide", p.rotate_z);
		e.commit();
	}

	/**
	 *  Loads the {@link android.opengl.GLSurfaceView GlSurface},
	 *  navigation buttons defined in main.xml, {@link OnClickListener and their functions}
	 * 
	 */
	private void loadView()
	{
		try{
			setContentView(R.layout.main);
			glSurfaceView= (GLSurfaceView) findViewById(R.id.map_surface);
			glSurfaceView.setRenderer(new IndMapRenderer(world));

			//glSurfaceView.setRenderMode(GLSurfaceView.RENDERMODE_WHEN_DIRTY);

			Button up1,left1,go1,right1,down1,up2,left2,go2,right2,down2;

			up1		= (Button) findViewById(R.id.up1);
			left1	= (Button) findViewById(R.id.left1);
			go1		= (Button) findViewById(R.id.go1);
			right1	= (Button) findViewById(R.id.right1);
			down1	= (Button) findViewById(R.id.down1);

			up2		= (Button) findViewById(R.id.up2);
			left2	= (Button) findViewById(R.id.left2);
			go2		= (Button) findViewById(R.id.go2);
			right2	= (Button) findViewById(R.id.right2);
			down2	= (Button) findViewById(R.id.down2);

			go1.setOnClickListener(new OnClickListener() {public void onClick(View v)
			{
				switch(buttonMode)
				{
					case MODE_ROT:
						world.toggleNodeMode();
						move(World.MOVE_UPD);
						break;
					case MODE_Z:
						world.clearAllAt();
						move(World.MOVE_UPD);
						break;
				}
			}});
			up1.setOnClickListener(new OnClickListener() {public void onClick(View v){
				move(World.MOVE_FWD);						
			}});
			down1.setOnClickListener(new OnClickListener() {public void onClick(View v){
				move(World.MOVE_BWD);
			}});

			left1.setOnClickListener(new OnClickListener() {public void onClick(View v){
				move(World.MOVE_LEFT);
			}});
			right1.setOnClickListener(new OnClickListener() {public void onClick(View v){
				move(World.MOVE_RIGHT);
			}});
			go2.setOnClickListener(new OnClickListener() {public void onClick(View v){
				switch(buttonMode)
				{
					case MODE_ROT:
						buttonMode=MODE_Z;
						break;
					case MODE_Z:
						buttonMode=MODE_ROT;
						break;
				}
				move(World.MOVE_UPD);
			}});
			up2.setOnClickListener(new OnClickListener() {public void onClick(View v){
				switch(buttonMode)
				{
					case MODE_ROT: move(World.MOVE_ROT_UP);break;
					case MODE_Z: move(World.MOVE_UP);break;
				}
			}});
			down2.setOnClickListener(new OnClickListener() {public void onClick(View v){
				switch(buttonMode)
				{
					case MODE_ROT: move(World.MOVE_ROT_DOWN);break;
					case MODE_Z: move(World.MOVE_DOWN);break;
				}
			}});
			left2.setOnClickListener(new OnClickListener() {public void onClick(View v){
				switch(buttonMode)
				{
					case MODE_ROT: move(World.MOVE_ROT_LEFT);break;
					case MODE_Z: world.scale=world.scale*0.77f;move(World.MOVE_UPD);break;
				}
			}});
			right2.setOnClickListener(new OnClickListener() {public void onClick(View v){
				switch(buttonMode)
				{
					case MODE_ROT: move(World.MOVE_ROT_RIGHT);break;
					case MODE_Z: world.scale=world.scale*1.3f;move(World.MOVE_UPD);break;
				}
			}});
			move(World.MOVE_UPD);
			glSurfaceView.requestRender();
			glSurfaceView.requestFocus();
		}catch (Exception e) {
			IndMap.log(IndMap.ERROR, TAG, "LoadView "+e.toString());
		}
	}

	/**
	 * Request a new {@link List} of link ScanResult}.
	 */
	private void scan()
	{
		receive_for=WIFI_WIFI;
		wiMan.startScan();
	}

	/**
	 * Calls {@link World#move(int)} then updates the positions info in the sidebar.
	 * 
	 * @param move Defines requested action, as of {@link World#MOVE_FWD MOVE_FWD}, {@link World#MOVE_BWD MOVE_BWD}, {@link World#MOVE_UP MOVE_UP}, 
	 * {@link World#MOVE_DOWN}, {@link World#MOVE_LEFT}, {@link World#MOVE_RIGHT}, {@link World#MOVE_ROT_UP}, 
	 * {@link World#MOVE_ROT_DOWN}, {@link World#MOVE_ROT_LEFT}, {@link World#MOVE_ROT_RIGHT}
	 */
	private void move(int move)
	{
		if(move!=World.MOVE_UPD)world.move(move,getApplicationContext());

		RotatePoint p=world.getCamera();
		TextView t=(TextView) findViewById(R.id.pos);
		t.setText("POS:\n  ("+(int)p.x+", "+(int)p.y+", "+(int)p.z+")");
		t=(TextView) findViewById(R.id.rot);
		t.setText("Rot: ("+(int)p.rotate_x+", "+(int)p.rotate_y+", "+(int)p.rotate_z+")");
		t=(TextView) findViewById(R.id.mode);
		t.setText("Modes:\n"+
				"-NavButMode: "+getResources().getStringArray(R.array.but_mode)[buttonMode]+"\n"+
				"-NodeMode: "+world.isMode_node()+"\n"+
				"-2D Mode: "+world.isMode_draw2D()
		);

		t=(TextView) findViewById(R.id.build_scale);
		t.setText("Free walk step: "+world.scale);

		t=(TextView) findViewById(R.id.info);
		StringBuilder sb=new StringBuilder();
		Route atRo=world.getAtRoute();
		Node atN=world.getAtNode();
		Room atR=world.getAtRoom();
		Floor atF=world.getAtFloor();
		Building atB=world.getAtBuilding();
		if(atRo!=null)sb.append("\n-Route: "+atRo.toString());
		if(atN!=null)sb.append("\n-Node: "+atN.toString());
		if(atR!=null)sb.append("\n-Room: "+atR.toString());
		if(atF!=null)sb.append("\n-Floor: "+atF.toString());
		if(atB!=null)sb.append("\n-Building: "+atB.toString());
		t.setText(sb.toString());

		glSurfaceView.requestRender();
	}

	/**
	 * Handles results from called Activities, such as {@link NodeActivity}.
	 *
	 * @param requestCode the request code
	 * @param resultCode the result code
	 * @param data the data
	 * @see android.app.Activity#onActivityResult(int, int, android.content.Intent)
	 */
	@Override
	public void onActivityResult(int requestCode, int resultCode, Intent data)
	{
		Uri uri;
		switch (requestCode)
		{
			case 1234:
				switch (resultCode)
				{
					case RESULT_OK:
						log(DEBUG, TAG, "BACK: Reloading world because of return from NodeActivity..");
						world.loadFromInternal(this); //Reload edited nodes, basicly reload everything
						if(data!=null&&(uri=data.getData())!=null&&uri.getPathSegments().size()>1)
						{
							if(uri.getPathSegments().get(0).equals("node"))world.setAtNode(Integer.parseInt(uri.getPathSegments().get(1)));
						}
						log(DEBUG, TAG, "..done");						
						break;
					case RESULT_FIRST_USER:
						log(DEBUG, TAG, "BACK: Got some stuff.. "+data.getExtras().toString());
						if(data!=null)checkBundle(data.getExtras());		
						break;
					default:
						log(DEBUG, TAG, "BACK: Nothing saved in NodeActivity");
						break;
				}
				break;
			case 4321:
				if(data!=null)checkBundle(data.getExtras());
				else log(ERROR, TAG, "BACK: Result expected from NodeActivity");
				break;
		}
	}

	/**
	 * Creates options menu.
	 *
	 * @param menu the menu
	 * @return true, if successful
	 * @see android.app.Activity#onCreateOptionsMenu(android.view.Menu)
	 */
	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		MenuInflater inflater = getMenuInflater();
		inflater.inflate(R.menu.option_menu, menu);
		return true;
	}

	/**
	 * Handles item selected in options menu.
	 *
	 * @param item the item
	 * @return true, if successful
	 * @see android.app.Activity#onOptionsItemSelected(android.view.MenuItem)
	 */
	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		// Handle item selection
		switch (item.getItemId())
		{
			case R.id.menu_test:
				try
				{
					world.moveTo(new RotatePoint(500f,200f,1000f,0.0f,-90f,-90f), World.TRANS_ALL);
				}
				catch (Exception e) {
					log(ERROR, TAG, "TEST "+e.toString());
				}
				return true;
			case R.id.menu_export:
				export();
				return true;
			case R.id.menu_getpos:
				scan();
				return true;
			case R.id.menu_editnodes:
				Intent edNoInt=new Intent(getApplicationContext(), NodeActivity.class);
				edNoInt.setAction(NodeActivity.EDIT_NODE_ACTION);
				Bundle b =new Bundle();
				if(world.getAtBuilding()!=null)	b.putInt(Building.PARENT, world.getAtBuilding().getI());
				if(world.getAtFloor()!=null)	b.putInt(Floor.PARENT, world.getAtFloor().getI());
				if(world.getAtRoom()!=null)		b.putInt(Room.PARENT, world.getAtRoom().getI());

				Node n;
				if((n=world.getAtNode())!=null&&world.isMode_node())
				{
					edNoInt.setData(Uri.withAppendedPath(Provider.CONTENT_URI, Node.TABLE_NAME+"/"+n.getI()));
					b.putInt("atNode", n.getI());
				}
				else
				{
					Point p=world.getCamera();
					edNoInt.setData(Uri.withAppendedPath(Provider.CONTENT_URI, Node.TABLE_NAME));
					b.putFloat(Node.X, p.x);
					b.putFloat(Node.Y, p.y);
					b.putFloat(Node.Z, p.z);
				}				
				edNoInt.putExtras(b);
				startActivityForResult(edNoInt, 1234);
				return true;
			case R.id.menu_download:
				loadFromExternalDialog("This will vipe and reload all data from external.");
				return true;
			case R.id.menu_sidebar:
				LinearLayout r= (LinearLayout) findViewById(R.id.sidebar);
				switch(r.getVisibility())
				{
					case View.VISIBLE:
						r.setVisibility(View.GONE);
						break;
					default:
						r.setVisibility(View.VISIBLE);
				}
				//				sidebar=(short) r.getVisibility();
				return true;
			case R.id.menu_settings:
				final Dialog d=new Dialog(IndMap.this);
				d.setContentView(R.layout.main_settings);
				d.setTitle(R.string.menu_settings);
				final EditText ext=(EditText) d.findViewById(R.id.ext);
				ext.setText(myphp.getURL());

				//SETTINGS SAVE
				final Button save=(Button) d.findViewById(R.id.save);
				save.setOnClickListener(new OnClickListener()
				{
					public void onClick(View v)
					{
						String url=ext.getText().toString();
						if(url==null||url.equals(""))
						{
							Toast.makeText(IndMap.this, "Url null or not set", Toast.LENGTH_LONG).show();
							return;
						}
						myphp.setURL(url);

						d.dismiss();
						Toast.makeText(IndMap.this, "Settings saved", Toast.LENGTH_LONG).show();
					}
				});

				final Button canc=(Button) d.findViewById(R.id.cancel);
				canc.setOnClickListener(new OnClickListener()
				{
					public void onClick(View v){d.dismiss();}
				});

				d.show();
				return true;
			case R.id.menu_2dmode:
				world.toggle2DMode();
				move(World.MOVE_UPD);
				return true;

			case R.id.menu_nodemode:
				world.toggleNodeMode();
				move(World.MOVE_UPD);
				return true;
			default:
				return super.onOptionsItemSelected(item);
		}
	}

	/**
	 * For handeling touch events. NOT IMPLEMENTED
	 *
	 * @param event the event
	 * @return true, if successful
	 * @see android.app.Activity#onTouchEvent(android.view.MotionEvent)
	 */
	@Override
	public boolean onTouchEvent(MotionEvent event) {
		switch (event.getAction()) {
			case MotionEvent.ACTION_MOVE:
				move(event.getX() - touchStart.x, touchStart.y - event.getY());
				touchStart.set(event.getX(), event.getY());
				glSurfaceView.requestRender();
				return true;

			case MotionEvent.ACTION_DOWN:
				touchStart.set(event.getX(), event.getY());
				return true;

			default:
				return super.onTouchEvent(event);
		}
	}

	/**
	 * Track key events.
	 *
	 * @param keyCode the key code
	 * @param event the event
	 * @return true, if successful
	 * @see android.app.Activity#onKeyDown(int, android.view.KeyEvent)
	 */
	@Override
	public boolean onKeyDown(int keyCode, KeyEvent event)
	{
		/*if(event.getAction()==KeyEvent.ACTION_DOWN)*/
		//int num=0;
		switch (keyCode) {
			case KeyEvent.KEYCODE_DPAD_UP:
				move(World.MOVE_FWD);
				return true;

			case KeyEvent.KEYCODE_DPAD_DOWN:
				move(World.MOVE_BWD);
				return true;

			case KeyEvent.KEYCODE_DPAD_LEFT:
				move(World.MOVE_LEFT);
				return true;

			case KeyEvent.KEYCODE_DPAD_RIGHT:
				move(World.MOVE_RIGHT);
				return true;

			case KeyEvent.KEYCODE_W: move(World.MOVE_FWD);return true;
			case KeyEvent.KEYCODE_S: move(World.MOVE_BWD);return true;
			case KeyEvent.KEYCODE_A: move(World.MOVE_LEFT);return true;
			case KeyEvent.KEYCODE_D: move(World.MOVE_RIGHT);return true;

			case KeyEvent.KEYCODE_I: move(World.MOVE_ROT_UP);return true;
			case KeyEvent.KEYCODE_K: move(World.MOVE_ROT_DOWN);return true;
			case KeyEvent.KEYCODE_J: move(World.MOVE_ROT_LEFT);return true;
			case KeyEvent.KEYCODE_L: move(World.MOVE_ROT_RIGHT);return true;

			case KeyEvent.KEYCODE_E: move(World.MOVE_UP);return true;
			case KeyEvent.KEYCODE_Q: move(World.MOVE_DOWN);return true;

			/*
			case KeyEvent.KEYCODE_N:
				num=world.getBuildings()[0].num_vFloor;
				if(num>0)world.getBuildings()[0].num_vFloor--;
				move(World.MOVE_UPD);
				return true;
			case KeyEvent.KEYCODE_M:
				num=world.getBuildings()[0].num_vFloor;
				world.getBuildings()[0].num_vFloor++;
				move(World.MOVE_UPD);
				return true;*/

			default:
				return super.onKeyDown(keyCode, event);      
		}
		//else return super.onKeyDown(keyCode, event);
	}

	/**
	 * For moving support for touch events. NOT IMPLEMENTED
	 *
	 * @param i ?
	 * @param j ?
	 */
	private void move(float i, float j)
	{
		glSurfaceView.requestRender();
	}


	/** Log code DEBUG used as first parameter when calling {@link #log(short, String, String)}. */
	public static final short DEBUG=1;

	/** Log code INFO used as first parameter when calling {@link #log(short, String, String)}. */
	public static final short INFO=2;

	/** Log code WARNING used as first parameter when calling {@link #log(short, String, String)}. */
	public static final short WARNING=3;

	/** Log code ERROR used as first parameter when calling {@link #log(short, String, String)}. */
	public static final short ERROR=4;

	/**
	 * Giving all entries in the log (access by adb logcat) the prefix IndMap, only requiering one logcat filter.
	 *
	 * @param CODE One of {@link #DEBUG}, {@link #INFO}, {@link #WARNING} or {@link #ERROR}
	 * @param tag The subactivity's TAG
	 * @param msg Log message
	 * 
	 * @see Log
	 */
	public static void log(final short CODE,String tag,String msg)
	{
		switch(CODE)
		{
			case DEBUG: Log.d(TAG,tag+" : "+msg);break;
			case INFO: Log.i(TAG,tag+" : "+msg);break;
			case WARNING: Log.w(TAG,tag+" : "+msg);break;
			case ERROR:  Log.e(TAG,tag+" : "+msg);break;
			default:  Log.v(TAG,tag+" : "+msg);
		}
	}

	/** Identifying the load from external storage ({@link MysqlViaPHP}) dialog id. (Progcess bar ect) */
	private static final int DIALOG_FROMEXTERNAL = 100; 

	/**
	 * Check bundle.
	 *
	 * @param b the b
	 */
	private void checkBundle(Bundle b)
	{
		IndMap.log(IndMap.DEBUG, TAG, "BACK: Checking Bundle sent from NodeActivity");
		if(b!=null)
		{
			IndMap.log(IndMap.DEBUG, TAG, "BACK: - Bundle found has "+b.size()+" keys. "+b.toString());
			try{
				if(b.containsKey("viewNode"))
				{
					IndMap.log(IndMap.DEBUG, TAG, "BACK:   - viewNode: "+b.get("viewNode").getClass().getName()+" Int_value: "+b.getInt("viewMode"));
					Node n= (Node) Node.searchById(world.getNodes(), b.getInt("viewNode"));
					if(n!=null)
					{
						IndMap.log(IndMap.DEBUG, TAG, "BACK:   - FOUND!! "+n.toString());
						Route atR=world.getAtRoute();
						if(atR!=null&&atR.isMember(n))world.setAtRoute(null);
						world.setAtNode(n);
					}
				}
				else if(b.containsKey("asRoute"))
				{
					world.setAtRoute(new Route(world.getNodes(), b.getIntArray("asRoute"), false));
				}
				else if(b.containsKey("makeRoute"))
				{
					world.setAtRoute(new Route(world.getNodes(), b.getIntArray("makeRoute"), true));
				}

			}catch (Exception e) {
				IndMap.log(IndMap.ERROR, TAG, "Checking Bundle "+e.toString());
			}
		}		
	}

	/**
	 * Makes an {@link AlertDialog} confirming to wipe internal data by downloading external data.
	 *
	 * @param msg The alert message.
	 */
	protected void loadFromExternalDialog(String msg)
	{
		AlertDialog.Builder builder = new AlertDialog.Builder(IndMap.this);
		builder.setMessage(msg+" Continue?")
		.setCancelable(false)
		.setPositiveButton("Yes", new DialogInterface.OnClickListener()
		{
			public void onClick(DialogInterface dialog, int id)
			{
				showDialog(DIALOG_FROMEXTERNAL);
				dialog.cancel();
			}
		})
		.setNegativeButton("No", new DialogInterface.OnClickListener()
		{

			public void onClick(DialogInterface dialog, int which)
			{
				dialog.cancel();
			}
		});
		AlertDialog alert = builder.create();
		alert.show();
	}

	/**
	 * For displaying different dialogs
	 * 
	 * <br>Until now implemented:
	 * <ul>
	 * <li> {@link World.LFEThread#run() Load from external} dialog ({@link #loadFromExternalDialog(String) pre-alert})</li>
	 * </ul>
	 *
	 * @param id the id
	 * @param args the args
	 * @return the dialog
	 * @see android.app.Activity#onCreateDialog(int, android.os.Bundle)
	 */
	@Override
	protected Dialog onCreateDialog(int id, Bundle args) 
	{
		switch(id) {
			case DIALOG_FROMEXTERNAL:
				progressDialog = new ProgressDialog(IndMap.this);
				progressDialog.setTitle("Load from external");
				progressDialog.setMessage("");
				progressDialog.setProgressStyle(ProgressDialog.STYLE_HORIZONTAL);
				return progressDialog;

			default:
				return null;
		}
	}

	/* (non-Javadoc)
	 * @see android.app.Activity#onPrepareDialog(int, android.app.Dialog)
	 */
	@Override
	protected void onPrepareDialog(int id, Dialog dialog)
	{
		switch(id)
		{
			case DIALOG_FROMEXTERNAL:
				progressDialog.setProgress(0);
				lfeThread = world.getLFEThread(myphp, this, hLoadExt);
				lfeThread.start();
				break;
		}
	}

	/**
	 * Handling updates from the  {@link World.LFEThread  load from external thread}  to be displayed in the loading dialog
	 * @uml.property  name="hLoadExt"
	 * @uml.associationEnd  multiplicity="(1 1)"
	 */
	final Handler hLoadExt = new Handler()
	{
		public void handleMessage(Message msg)
		{
			int total=msg.arg1;
			progressDialog.setMessage((String)msg.obj);
			progressDialog.setProgress(total);
			if (total > 100)dismissDialog(DIALOG_FROMEXTERNAL);
			if(((String)msg.obj).equals("ERROR!"))
			{
				AlertDialog.Builder builder = new AlertDialog.Builder(IndMap.this);
				builder.setTitle("Error")
				.setMessage("An error occured while loading. Retry?")
				.setCancelable(false)
				.setPositiveButton("Yes", new DialogInterface.OnClickListener()
				{
					public void onClick(DialogInterface dialog, int id)
					{
						dismissDialog(DIALOG_FROMEXTERNAL);
						showDialog(DIALOG_FROMEXTERNAL);
						dialog.cancel();
					}
				})
				.setNegativeButton("No", new DialogInterface.OnClickListener()
				{
					public void onClick(DialogInterface dialog, int id) 
					{
						dismissDialog(DIALOG_FROMEXTERNAL);
						dialog.cancel();
					}
				});
				builder.create().show();;
			}
		}
	};

	/**
	 * Display the user the message "Not yet implemented" where needed.
	 */
	protected void notImplemented()
	{
		Toast.makeText(this, "Not yet implemented", Toast.LENGTH_LONG).show();
	}

	/**
	 * Export nodes, aps, and relations to a file.
	 * 
	 * @see World#export(Context)
	 */
	protected void export()
	{
		world.export(getApplicationContext());
	}
}