/*********************************************************************************************
 * File:    Node.java
 * Project: Indoor maps, Masterthesis
 * Version: 0.5b
 * Author:  Jon Villy Meidell
 * Org:     NTNU
 * 
 * 
 * Data Structure
 * A Node / Navigation point
 */
package no.ntnu.map.indoormaps;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.security.InvalidParameterException;
import java.util.ArrayList;

import javax.microedition.khronos.opengles.GL10;

import no.ntnu.map.indoormaps.World.Colour;
import no.ntnu.map.indoormaps.Storage.MysqlViaPHP;
import no.ntnu.map.indoormaps.Storage.SQLStorage.QueryBuilder;
import no.ntnu.map.indoormaps.Storage.SQLStorage.QueryBuilder.WrongTypeException;

import org.json.JSONObject;

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.graphics.Color;
import android.net.Uri;

// TODO: Auto-generated Javadoc
/**
 * The Class Node.
 */
public class Node extends AbstractEntety
{
	/** TAG used with {@link IndMap#log(short, String, String)}. */
	public static final String TAG="Node";

	/** The Database table name. Used in {@link Provider} and calls to it, {@link QueryBuilder}, ect*/
	public static final String 	TABLE_NAME="node";

	/** The parent key as foreign key. */
	public static final String PARENT="node_id";

	/** The parent key as foreign key #1 in nodenode table. */
	public static final String PARENTA="a_id";

	/** The parent key as foreign key #2 in nodenode table. */
	public static final String PARENTB="b_id";

	/** The Constant DESC. */
	public static final String DESC="descr";

	/** The Constant X. */
	public static final String X="x";

	/** The Constant Y. */
	public static final String Y="y";

	/** The Constant Z. */
	public static final String Z="z";

	/** The Constant CHILD_COST. */
	public static final String CHILD_COST="cost";

	/** The draw node count. */
	public static int drawNodeCount=0;

	/** The COL_DEFAULT. */
	public static final Colour COL_AT_NODE=new Colour(0, 50, 50, 40);

	/** The COL_DEFAULT. */
	public static final Colour	COL_AT_ROUTE=new Colour(0, 0, 200, 160);

	/** The COL_DEFAULT. */
	public static final Colour	COL_DEFAULT=new Colour(0, 150, 150, 180);
	
	/** The COL_DEFAULT. */
	public static final Colour	COL_NEXT_NODE=new Colour(0,50,100,255);

	/** The Constant MAX_NUM_APS. */
	public static final short MIN_SIGNAL=-100; //TODO Adjust min sigal og max num of aps pr node

	/** The Constant MAX_NUM_APS. */
	public static final short MAX_NUM_APS=10;

	/** The Constant FROM_FLOOR. */
	public static final short FROM_FLOOR=(short)World.FROM_FLOOR/3;

	/**
	 * The rotate.
	 * @uml.property  name="rotate"
	 */
	private int rotate=0;

	/**
	 * The Interface NodeSQLEntety.
	 */
	public static interface NodeSQLEntety extends SQLEntety
	{
		/**
		 * Gets the by building.
		 *
		 * @param parent_id the parent_id
		 * @return the by building
		 * @throws WrongTypeException the wrong type exception
		 */
		public String getByBuilding(int parent_id) throws WrongTypeException;

		/**
		 * Gets the by floor.
		 *
		 * @param parent_id the parent_id
		 * @return the by floor
		 * @throws WrongTypeException the wrong type exception
		 */
		public String getByFloor(int parent_id) throws WrongTypeException;

		/**
		 * Gets the by room.
		 *
		 * @param parent_id the parent_id
		 * @return the by room
		 * @throws WrongTypeException the wrong type exception
		 */
		public String getByRoom(int parent_id) throws WrongTypeException;

		/**
		 * Gets the cost.
		 *
		 * @param parent1 the parent1
		 * @param parent2 the parent2
		 * @return the cost
		 * @throws WrongTypeException the wrong type exception
		 */
		public String getCost(int parent1, int parent2) throws WrongTypeException;
	}

	/**
	 * The Class Entety.
	 */
	public static class Entety implements NodeSQLEntety
	{

		/** The Constant PROJ_ALL. */
		public static final String[] PROJ_ALL={_ID,NAME,DESC,Room.PARENT,Floor.PARENT,Building.PARENT,X,Y,Z};

		/* (non-Javadoc)
		 * @see no.ntnu.map.indoormaps.AbstractEntety.SQLEntety#createTable()
		 */
		public String createTable() throws WrongTypeException
		{
			QueryBuilder qb=new QueryBuilder(QueryBuilder.CREATE);
			qb.setTable(TABLE_NAME);
			qb.addField(_ID,         "INTEGER", true);
			qb.addField(NAME,        "TEXT");
			qb.addField(DESC,        "TEXT");
			qb.addField(Room.PARENT, "INTEGER NULL DEFAULT NULL");
			qb.addField(Floor.PARENT,"INTEGER NULL DEFAULT NULL");
			qb.addField(Building.PARENT, "INTEGER NULL DEFAULT NULL");
			qb.addField(X,           "REAL");
			qb.addField(Y,           "REAL");
			qb.addField(Z,           "REAL");
			String ret=qb.getSqlString();
			qb=new QueryBuilder(QueryBuilder.CREATE);
			qb.setTable(TABLE_NAME+TABLE_NAME);
			qb.addField(PARENTA,"INTEGER", true);
			qb.addField(PARENTB,"INTEGER", true);
			qb.addField(CHILD_COST, "REAL");
			ret+=qb.getSqlString();
			return ret;
		}

		/* (non-Javadoc)
		 * @see no.ntnu.map.indoormaps.AbstractEntety.SQLEntety#dropTable()
		 */
		public String dropTable() throws WrongTypeException
		{
			return SQLEntety.DROPTABLESQL+TABLE_NAME;
		}

		/* (non-Javadoc)
		 * @see no.ntnu.map.indoormaps.AbstractEntety.SQLEntety#getAll()
		 */
		public String getAll() throws WrongTypeException {
			//Generate SQL-query to locate all nodes
			QueryBuilder qb=new QueryBuilder(QueryBuilder.SELECT);
			qb.addTable(TABLE_NAME);
			qb.addProjection(PROJ_ALL);
			qb.addOrdering(_ID);
			return qb.getSqlString();
		}

		/* (non-Javadoc)
		 * @see no.ntnu.map.indoormaps.AbstractEntety.SQLEntety#getByParent(no.ntnu.map.indoormaps.AbstractEntety)
		 */
		public String getByParent(AbstractEntety parent) throws WrongTypeException
		{
			if(!(parent instanceof Node))throw new InvalidParameterException("A Node parent is supposed to be a Node");
			return getByParent(parent.getI());
		}

		/* (non-Javadoc)
		 * @see no.ntnu.map.indoormaps.AbstractEntety.SQLEntety#getByParent(int)
		 */
		public String getByParent(int parent_id) throws WrongTypeException {
			//Generate SQL-query to locate all nodes by node
			QueryBuilder qb1,qb2=new QueryBuilder(QueryBuilder.SELECT,false);
			qb2.addTable(TABLE_NAME);
			qb2.addJoinCond(_ID, QueryBuilder.EQUALS, PARENTA);
			qb2.addTable(TABLE_NAME+TABLE_NAME);
			qb2.addProjection(PROJ_ALL);
			qb2.addProjection(CHILD_COST);
			qb2.addWhereCond(Node.PARENTB, QueryBuilder.EQUALS, parent_id);

			qb1=new QueryBuilder(QueryBuilder.SELECT,false);
			qb1.addTable(TABLE_NAME);
			qb1.addJoinCond(_ID, QueryBuilder.EQUALS, PARENTB);
			qb1.addTable(TABLE_NAME+TABLE_NAME);
			qb1.addProjection(PROJ_ALL);
			qb1.addProjection(CHILD_COST);
			qb1.addWhereCond(Node.PARENTA, QueryBuilder.EQUALS, parent_id);

			return ""+qb2.getSqlString()+" UNION "+qb1.getSqlString()+" ORDER BY _ID;";
		}

		/* (non-Javadoc)
		 * @see no.ntnu.map.indoormaps.AbstractEntety.SQLEntety#getOne(int)
		 */
		public String getOne(int id) throws WrongTypeException
		{
			//Generate SQL-query to a node by id
			QueryBuilder qb=new QueryBuilder(QueryBuilder.SELECT);
			qb.addTable(TABLE_NAME);
			qb.addProjection(PROJ_ALL);
			qb.addOrdering(_ID);
			qb.addWhereCond(_ID, QueryBuilder.EQUALS, id);
			return qb.getSqlString();
		}

		/* (non-Javadoc)
		 * @see no.ntnu.map.indoormaps.Node.NodeSQLEntety#getByBuilding(int)
		 */
		public String getByBuilding(int parent_id) throws WrongTypeException
		{
			QueryBuilder qb=new QueryBuilder(QueryBuilder.SELECT);
			qb.addTable(TABLE_NAME);
			qb.addProjection(PROJ_ALL);
			qb.addOrdering(_ID);
			if(parent_id==0)qb.addWhereNull(Building.PARENT);
			else qb.addWhereCond(Building.PARENT, QueryBuilder.EQUALS, parent_id);
			return qb.getSqlString();
		}

		/* (non-Javadoc)
		 * @see no.ntnu.map.indoormaps.Node.NodeSQLEntety#getByFloor(int)
		 */
		public String getByFloor(int parent_id) throws WrongTypeException
		{
			QueryBuilder qb=new QueryBuilder(QueryBuilder.SELECT);
			qb.addTable(TABLE_NAME);
			qb.addProjection(PROJ_ALL);
			qb.addOrdering(_ID);
			qb.addWhereCond(Floor.PARENT, QueryBuilder.EQUALS, parent_id);
			return qb.getSqlString();
		}

		/* (non-Javadoc)
		 * @see no.ntnu.map.indoormaps.Node.NodeSQLEntety#getByRoom(int)
		 */
		public String getByRoom(int parent_id) throws WrongTypeException
		{
			QueryBuilder qb=new QueryBuilder(QueryBuilder.SELECT);
			qb.addTable(TABLE_NAME);
			qb.addProjection(PROJ_ALL);
			qb.addOrdering(_ID);
			qb.addWhereCond(Room.PARENT, QueryBuilder.EQUALS, parent_id);
			//			IndMap.log(IndMap.DEBUG, TAG, "Node SQL: "+qb.getSqlString());
			return qb.getSqlString();
		}

		/* (non-Javadoc)
		 * @see no.ntnu.map.indoormaps.Node.NodeSQLEntety#getCost(int, int)
		 */
		public String getCost(int parent1, int parent2) throws WrongTypeException
		{
			QueryBuilder qb1,qb2=new QueryBuilder(QueryBuilder.SELECT,false);
			qb2.addTable(TABLE_NAME+TABLE_NAME);
			qb2.addProjection(CHILD_COST);
			qb2.addWhereCond(Node.PARENTB, QueryBuilder.EQUALS, parent1);
			qb2.addWhereCond(Node.PARENTA, QueryBuilder.EQUALS, parent2);

			qb1=new QueryBuilder(QueryBuilder.SELECT,false);
			qb1.addTable(TABLE_NAME+TABLE_NAME);
			qb1.addProjection(CHILD_COST);
			qb1.addWhereCond(Node.PARENTA, QueryBuilder.EQUALS, parent1);
			qb1.addWhereCond(Node.PARENTB, QueryBuilder.EQUALS, parent2);

			return qb2.getSqlString()+" UNION "+qb1.getSqlString();
		}
	}

	/* (non-Javadoc)
	 * @see no.ntnu.map.indoormaps.AbstractEntety#toInternal(android.content.Context)
	 */
	@Override
	void toInternal(Context c, int a)
	{
		//Insert node
		ContentValues values=new ContentValues();
		try{
			values.put(_ID, _id);
			//		values.put(ID,id);
			values.put(NAME,name);
			values.put(DESC, desc);
			if(room!=null)
			{		
				values.put(Room.PARENT, room.getI());
				values.put(Floor.PARENT, room.getFloor().getI());
				values.put(Building.PARENT, room.getBuilding().getI());
			}
			else if(floor!=null)
			{
				values.putNull(Room.PARENT);
				values.put(Floor.PARENT, floor.getI());
				values.put(Building.PARENT, floor.getBuilding().getI());
			}
			else
			{
				values.putNull(Room.PARENT);
				values.putNull(Floor.PARENT);
				values.putNull(Building.PARENT);
			}
			values.put(X, point.x);
			values.put(Y, point.y);
			values.put(Z, point.z);
			c.getContentResolver().insert(Uri.withAppendedPath(Provider.CONTENT_URI,TABLE_NAME), values);
		}catch (Exception e) {
			IndMap.log(IndMap.ERROR, TAG, "Error save to internal "+e.toString());
		}

		//Insert neightbors
		for(int i=0;i<neighbor.size();i++)
		{
			int nid=neighbor.get(i).getI();
			if(nid<_id)continue;
			values=new ContentValues();
			values.put(PARENTA, _id);
			values.put(PARENTB, nid);
			values.put(CHILD_COST, cost[i]);
			try{c.getContentResolver().insert(Uri.withAppendedPath(Provider.CONTENT_URI,TABLE_NAME+TABLE_NAME), values);}catch (Exception e) {
				IndMap.log(IndMap.ERROR, TAG, "toInternal-Neighbors "+e.toString());
			}
		}

		//Insert AP-profiles
		for(int i=0;i<aps.length;i++)
		{
			values=new ContentValues();
			values.put(Ap.PARENT, aps[i].getI());
			values.put(PARENT, _id);
			values.put(Ap.LEVELatNODE, level[i]);
			try{c.getContentResolver().insert(Uri.withAppendedPath(Provider.CONTENT_URI,Ap.TABLE_NAME+Node.TABLE_NAME), values);}catch (Exception e) {
				IndMap.log(IndMap.ERROR, TAG, "toInternal-AP-profiles "+e.toString());
			}
		}
	}

	/**
	 * The room.
	 * @uml.property  name="room"
	 * @uml.associationEnd  inverse="node:no.ntnu.map.indoormaps.Room"
	 */
	private Room room=null;

	private Floor floor=null;

	/**
	 * The point.
	 * @uml.property  name="point"
	 * @uml.associationEnd  multiplicity="(1 1)"
	 */
	private Point point=null;

	/**
	 * The desc.
	 * @uml.property  name="desc"
	 */
	private String desc=null;

	/**
	 * The neighbor.
	 * @uml.property  name="neighbor"
	 * @uml.associationEnd  multiplicity="(0 -1)" elementType="no.ntnu.map.indoormaps.Node"
	 */
	private ArrayList<Node> neighbor=new ArrayList<Node>();

	/**
	 * The cost.
	 * @uml.property  name="cost" multiplicity="(0 -1)" dimension="1"
	 */
	private float[] cost=new float[0];

	private float[] angles=new float[0];

	/**
	 * The aps.
	 * @uml.property  name="aps"
	 * @uml.associationEnd  multiplicity="(0 -1)" inverse="nodes:no.ntnu.map.indoormaps.Ap"
	 */
	private Ap[]  aps=new Ap[MAX_NUM_APS];

	/**
	 * The level.
	 * @uml.property  name="level" multiplicity="(0 -1)" dimension="1"
	 */
	private int[] level=new int[MAX_NUM_APS];

	/**
	 * Instantiates a new node.
	 *
	 * @param name the name
	 * @param description the description
	 * @param r the r
	 * @param p the p
	 */
	public Node(String _name,String description,Room r, Floor f,Point p)
	{
		name=_name;
		room=r;
		floor=f;
		point=p;
		desc=description;
	}

	/**
	 * Instantiates a new node.
	 *
	 * @param c A {@link Cursor} giving buildings from {@link Provider internal storage} 
	 */
	Node(Cursor c)
	{
		try{
			_id=c.getInt(c.getColumnIndex(_ID));
			//		id=c.getString(c.getColumnIndex(ID));
			name=c.getString(c.getColumnIndex(NAME));
			desc=c.getString(c.getColumnIndex(DESC));
			point=new Point(c.getFloat(c.getColumnIndex(X)),
					c.getFloat(c.getColumnIndex(Y)),
					c.getFloat(c.getColumnIndex(Z)));
			if(IndMap.LOAD_INT_DEBUG)IndMap.log(IndMap.DEBUG, TAG, "In.New "+toString());
		}catch (Exception e) {
			IndMap.log(IndMap.ERROR, TAG, "ERR "+e.toString());
		}

	}

	/**
	 * Sets the a psand levels.
	 *
	 * @param aps the aps
	 * @param level the level
	 */
	public void setAPsandLevels(Ap[] aps,int[] level)
	{
		this.aps=aps;
		this.level=level;
	}

	/**
	 * Gets the a ps.
	 *
	 * @return the a ps
	 */
	public Ap[] getAPs(){return aps;}

	/**
	 * Gets the aP levels.
	 *
	 * @return the aP levels
	 */
	public int[] getAPLevels(){return level;}

	/**
	 * Adds the neighbor.
	 *
	 * @param neighb the neighb
	 * @param cost the cost
	 */
	public void addNeighbor(Node neighb, float cost)
	{
		if(!neighbor.contains(neighb))
		{
			neighbor.add(neighb);
			neighb.addNeighbor(this, cost);
			float[] temp=new float[this.cost.length+1];
			for(int i=0;i<this.cost.length;i++)
			{
				temp[i]=this.cost[i];
			}
			temp[this.cost.length]=cost;
			this.cost=temp;
		}
	}

	/**
	 * Gets the room.
	 * @return  the room
	 * @uml.property  name="room"
	 */
	public Room getRoom(){return room;}

	/**
	 * Sets the room.
	 * @param r  the new room
	 * @uml.property  name="room"
	 */
	public void setRoom(Room r)
	{
		room=r;
		if(r.getNode()!=null && r.getNode().equals(this))return;
		r.setNode(this);
	}

	public Floor getFloor(){return floor;}

	public void setFloor(Floor f)
	{
		floor=f;
	}

	/**
	 * Gets the point.
	 * @return  the point
	 * @uml.property  name="point"
	 */
	public Point getPoint()
	{
		return point;
	}

	/**
	 * Gets the desc.
	 * @return  the desc
	 * @uml.property  name="desc"
	 */
	public String getDesc(){return desc;}

	/**
	 * Gets the cost.
	 *
	 * @param child the child
	 * @return the cost
	 */
	public float getCost(Node child){return cost[neighbor.indexOf(child)];}

	/**
	 * Number children.
	 *
	 * @return the int
	 */
	public int numberOfNeighbors(){return neighbor.size();}

	/**
	 * Gets the child.
	 *
	 * @param i the i
	 * @return the child
	 */
	public Node getNeighbor(int i){return neighbor.get(i);}

	public float getNeighborAngle(int i){
		return angles[i];}

	/**
	 * Sort neighbors.
	 */
	public void sortNeighborsCalcAngles()
	{
		if(neighbor.size()<1)return;
		Node[] neigh=new Node[neighbor.size()];
		float[] angle=new float[neighbor.size()];
		float laNeg=Float.NEGATIVE_INFINITY;
		float smPos=Float.POSITIVE_INFINITY;
		int laNegIndex=0,smPosIndex=0;

		Point a=point,b;
		for(int i=0;i<neighbor.size();i++)
		{
			float ang;
			b=neighbor.get(i).getPoint();
			if(a.x==b.x&&a.y==b.y)
			{
				if(a.z==b.z)ang=Float.POSITIVE_INFINITY; //Should never happen, but just in case
				else
				{
					double abdx=b.x-a.x,abdy=b.y-a.y;
					float xy_axis=(float)Math.sqrt(abdx*abdx+abdy*abdy);
					ang=90f - Point.calcAngle(a.z, 0.0f,b.z, xy_axis);
				}
			}
			else ang=Point.calcAngle(a.x, a.y, b.x, b.y);
			if(ang<0f&&ang>laNeg)
			{
				laNeg=ang;
				laNegIndex=i;
			}
			if(ang>=0f&&ang<smPos)
			{
				smPos=ang;
				smPosIndex=i;
			}
			if(ang<0f)ang+=360f;
			angle[i]=ang;
		}
		int done=0;
		angles=new float[neigh.length];
		if(smPos+laNeg<0.0f)
		{
			neigh[0]=neighbor.get(smPosIndex);
			angles[0]=angle[smPosIndex];
			done=smPosIndex;
		}
		else
		{
			neigh[0]=neighbor.get(laNegIndex);
			angles[0]=angle[laNegIndex];
			done=laNegIndex;
		}
		int n=1;
		int[] sorted=AbstractEntety.sortGetSortetIndexes(angle);
		for(int i=0;i<sorted.length;i++)
		{
			if(sorted[i]==done)continue;
			angles[n]=angle[sorted[i]];
			neigh[n++]=neighbor.get(sorted[i]);
		}
		neighbor=new ArrayList<Node>(neigh.length);
		for(int i=0;i<neigh.length;i++)neighbor.add(neigh[i]);
	}

	/**
	 * Instantiates a new node.
	 *
	 * @param json the json
	 */
	Node(JSONObject json)
	{
		try
		{
			//Generate itself from the result of a sql-query
			_id=json.getInt(_ID);
			//id=json.getString(ID);
			name=json.getString(NAME);
			desc=json.getString(DESC);
			point=new Point(MysqlViaPHP.toFloat(json.getDouble(X)), MysqlViaPHP.toFloat(json.getDouble(Y)), MysqlViaPHP.toFloat(json.getDouble(Z)));
			if(IndMap.LOAD_EX_DEBUG)IndMap.log(IndMap.DEBUG, TAG, "Ex.New "+toString());
		}
		catch (Exception e) {
			IndMap.log(IndMap.ERROR, TAG, "Error conv from JSON to this "+TAG+""+e.toString());
			return;
		}
	}

	/**
	 * Gets the shortes path to.
	 *
	 * @param end the end
	 * @return the shortes path to
	 */
	public ArrayList<Node> getShortesPathTo(Node end)
	{
		return null;
	}

	/* (non-Javadoc)
	 * @see no.ntnu.map.indoormaps.AbstractEntety#draw(javax.microedition.khronos.opengles.GL10, no.ntnu.map.indoormaps.World)
	 */
	@Override
	public void draw(GL10 gl,World w)
	{
		try
		{
			Colour c;

			gl.glPushMatrix();

			Node at=w.getAtNode();
			if(
					(w.getAtRoute()==null&&!w.isMode_draw2D()) ||
					(w.isMode_draw2D()&&floor!=null&&floor.equals(w.getAtFloor()))
			)
			{
				if(equals(at))c=Route.HIGH_LIGHT;
				else c=Route.DEFAULT;
				draw(gl, GL10.GL_LINES, vFloorBuffer, num_vFloor, Color.argb(c.alpha, c.red, c.green, c.blue));
			}
			if(
					((w.getAtRoute()==null||(w.getAtRoute()!=null&&w.getAtRoute().isMember(this))) && !w.isMode_draw2D()) || 
					(w.isMode_draw2D()&&floor.equals(w.getAtFloor())))
			{
				c=COL_DEFAULT;
				if(equals(at)) c=COL_AT_NODE;
				else if(w.getAtRoute()!=null&&w.getAtRoute().isMember(this))c=COL_AT_ROUTE;
				else if(equals(w.getNextNode()))c=COL_NEXT_NODE;

				gl.glTranslatef(point.y, point.z+FROM_FLOOR, -point.x);
				gl.glRotatef(rotate, 0.5f, 1.0f, -0.2f);
				gl.glScalef(0.5f, 0.5f, 0.5f);

				draw(gl, GL10.GL_TRIANGLE_STRIP, vWallBuffer, num_vWall, Color.argb(c.alpha/(w.isMode_draw2D()?1:2), c.red, c.green, c.blue)); // Cube
				c=new Colour(255, 255, 255, c.alpha);
				if(!equals(at))draw(gl, GL10.GL_LINE_LOOP, vWallBuffer, num_vWall, Color.argb(c.alpha/(w.isMode_draw2D()?1:2), c.red, c.green, c.blue)); //Linjer p cube
			}
			gl.glPopMatrix();

			rotate+=5;
			if(rotate>360)rotate-=360;
		}catch (Exception e) {
			IndMap.log(IndMap.ERROR, TAG, "Draw "+e.toString());
		}
	}

	/* (non-Javadoc)
	 * @see no.ntnu.map.indoormaps.AbstractEntety#generateBuffers()
	 */
	public void generateBuffers()
	{
		int[] node = {
				//bak
				11,  11, -11, //v0
				11, -11, -11, //v1
				-11,  11, -11, //v2
				-11, -11, -11, //v3
				-11, -11,  11, //v4
				11, -11,  11, //v5
				11, -11, -11, //v1
				11,  11,  11, //v6
				11,  11, -11, //v0
				-11,  11,  11, //v7
				-11,  11, -11, //v2
				-11, -11,  11, //v4
				11,  11,  11, //v6
				11, -11,  11, //v5  14 vertexes	
		};
		num_vWall=node.length/3;
		ByteBuffer bb = ByteBuffer.allocateDirect(num_vWall*4*3);
		bb.order(ByteOrder.nativeOrder());
		vWallBuffer = bb.asIntBuffer();

		int i,n=0;
		for(i=0;i<node.length;i++)vWallBuffer.put(toFixed(node[i]));

		vFloor=new int[neighbor.size()*6];
		num_vFloor=neighbor.size()*2;
		for(i=0;i<neighbor.size();i++)
		{
			// X = y' Y = z' Z = -x'
			vFloor[n++]=toFixed(point.y);
			vFloor[n++]=toFixed(point.z+FROM_FLOOR);
			vFloor[n++]=toFixed(-point.x);

			Point p=neighbor.get(i).getPoint();
			vFloor[n++]=toFixed(p.y);
			vFloor[n++]=toFixed(p.z+FROM_FLOOR+1f);
			vFloor[n++]=toFixed(-p.x);
		}

		bb = ByteBuffer.allocateDirect(num_vFloor*4*3);
		bb.order(ByteOrder.nativeOrder());
		vFloorBuffer = bb.asIntBuffer();
		vFloorBuffer.put(vFloor);
	}

	/**
	 * Distance to.
	 *
	 * @param p the p
	 * @return the int
	 */
	public int distanceTo(Point p)
	{
		Point a=point;
		float dx=p.x-a.x, dy=p.y-a.y,dz=p.z-a.z;
		return (int) Math.sqrt(dx*dx+dy*dy+dz*dz);
	}
	public int distanceTo(Node n)
	{
		if(n==null)return Integer.MAX_VALUE;
		return distanceTo(n.getPoint());
	}

	public int distanceToNeighbor(int i)
	{
		if(i<0||i>=neighbor.size())return Integer.MAX_VALUE;
		return distanceTo(neighbor.get(i).getPoint());
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	public String toString()
	{
		return _id+": "+name+" "+point.toString();
	}
}
